package com.jarvis.dataCenter.controller;

import com.jarvis.dataCenter.config.OpenReplicatorCofig;
import com.jarvis.dataCenter.enums.SyncEnum;
import com.jarvis.dataCenter.runner.OpenReplicatorRunner;
import com.jarvis.dataCenter.service.impl.report.ThreadServiceImpl;
import com.jarvis.lib.common.base.message.Message;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;


/**
 * @author jinzhaopo
 * @date 2019-10-21 14:48
 */
@RestController
@Slf4j
@Api("SYNC")
public class SyncController {

    @Autowired
    private ThreadServiceImpl threadserviceImpl;

    @Autowired
    private OpenReplicatorRunner openReplicatorRunner;

    @Autowired
    private OpenReplicatorCofig openReplicatorCofig;


    /**
     * 查询控制同步的状态
     *
     * @param syncEnum
     * @return
     */
    @ApiOperation("获取状态信息")
    @GetMapping("/getSyncBol")
    public Message<String> getSyncBol(SyncEnum syncEnum) {
        Boolean status = null;
        if (syncEnum == SyncEnum.REPORT_SKU) {
            status = ThreadServiceImpl.syncSku;
        } else if (syncEnum == SyncEnum.REPORT_ADDRESS) {
            status = ThreadServiceImpl.syncAddress;
        } else if (syncEnum == SyncEnum.REPORT_AGENCY) {
            status = ThreadServiceImpl.syncAgency;
        } else if (syncEnum == SyncEnum.REPORT_CS) {
            status = ThreadServiceImpl.syncCsInfo;
        } else if (syncEnum == SyncEnum.REPORT_DO) {
            status = ThreadServiceImpl.sycnDoInfo;
        } else if (syncEnum == SyncEnum.REPORT_INVOICE) {
            status = ThreadServiceImpl.syncInvoiceInfo;
        } else if (syncEnum == SyncEnum.REPORT_SO) {
            status = ThreadServiceImpl.syncSoInfo;
        }

        return Message.success(status + "");
    }

    /**
     * @param syncEnum
     * @param status
     * @return
     */
    @ApiOperation("设置状态信息")
    @GetMapping("/setSyncBol")
    public Message<String> setSyncBol(SyncEnum syncEnum, @RequestParam boolean status) {
        if (syncEnum == SyncEnum.REPORT_SKU) {
            ThreadServiceImpl.syncSku = status;
        } else if (syncEnum == SyncEnum.REPORT_ADDRESS) {
            ThreadServiceImpl.syncAddress = status;
        } else if (syncEnum == SyncEnum.REPORT_AGENCY) {
            ThreadServiceImpl.syncAgency = status;
        } else if (syncEnum == SyncEnum.REPORT_CS) {
            ThreadServiceImpl.syncCsInfo = status;
        } else if (syncEnum == SyncEnum.REPORT_DO) {
            ThreadServiceImpl.sycnDoInfo = status;
        } else if (syncEnum == SyncEnum.REPORT_INVOICE) {
            ThreadServiceImpl.syncInvoiceInfo = status;
        } else if (syncEnum == SyncEnum.REPORT_SO) {
            ThreadServiceImpl.syncSoInfo = status;
        }

        return Message.success(status + "");
    }


    /**
     * @param syncEnum
     * @return
     */
    @ApiOperation("获取线程的状态")
    @GetMapping("/getThreadIsAlive")
    public Message<String> getThreadIsAlive(SyncEnum syncEnum) {

        boolean alive = false;
        if (syncEnum == SyncEnum.REPORT_SKU) {
            alive = threadserviceImpl.getSyncSkuThread().isAlive();
        } else if (syncEnum == SyncEnum.REPORT_ADDRESS) {
            alive = threadserviceImpl.getSyncAddressThread().isAlive();
        } else if (syncEnum == SyncEnum.REPORT_AGENCY) {
            alive = threadserviceImpl.getSyncAgencyThread().isAlive();
        } else if (syncEnum == SyncEnum.REPORT_CS) {
            alive = threadserviceImpl.getSyncCsInfoThread().isAlive();
        } else if (syncEnum == SyncEnum.REPORT_DO) {
            alive = threadserviceImpl.getSycnDoInfoThread().isAlive();
        } else if (syncEnum == SyncEnum.REPORT_INVOICE) {
            alive = threadserviceImpl.getSyncInvoiceInfoThread().isAlive();
        } else if (syncEnum == SyncEnum.REPORT_SO) {
            alive = threadserviceImpl.getSyncSoInfoThread().isAlive();

        }

        return Message.success(alive + "");
    }

    /**
     * 获取线程的睡眠时间
     *
     * @param syncEnum
     * @return
     */
    @ApiOperation("获取线程的睡眠时间")
    @GetMapping("/getSleepTime")
    public Message<String> getSleepTime(SyncEnum syncEnum) {
        long time = 0;
        if (syncEnum == SyncEnum.REPORT_SKU) {
            time = ThreadServiceImpl.sleepSku;
        } else if (syncEnum == SyncEnum.REPORT_ADDRESS) {
            time = ThreadServiceImpl.sleepAddress;
        } else if (syncEnum == SyncEnum.REPORT_AGENCY) {
            time = ThreadServiceImpl.sleepAgency;
        } else if (syncEnum == SyncEnum.REPORT_CS) {
            time = ThreadServiceImpl.sleepCsInfo;
        } else if (syncEnum == SyncEnum.REPORT_DO) {
            time = ThreadServiceImpl.sleepDoInfo;
        } else if (syncEnum == SyncEnum.REPORT_INVOICE) {
            time = ThreadServiceImpl.sleepInvoiceInfo;
        } else if (syncEnum == SyncEnum.REPORT_SO) {
            time = ThreadServiceImpl.sleepSoInfo;

        }
        return Message.success(time + "");
    }

    /**
     * 设置线程的睡眠时间
     *
     * @param syncEnum
     * @param time
     * @return
     */
    @ApiOperation("设置线程的睡眠时间")
    @GetMapping("/setSleepTime")
    public Message<String> getSleepTime(SyncEnum syncEnum, @RequestParam long time) {
        if (syncEnum == SyncEnum.REPORT_SKU) {
            ThreadServiceImpl.sleepSku = time;
        } else if (syncEnum == SyncEnum.REPORT_ADDRESS) {
            ThreadServiceImpl.sleepAddress = time;
        } else if (syncEnum == SyncEnum.REPORT_AGENCY) {
            ThreadServiceImpl.sleepAgency = time;
        } else if (syncEnum == SyncEnum.REPORT_CS) {
            ThreadServiceImpl.sleepCsInfo = time;
        } else if (syncEnum == SyncEnum.REPORT_DO) {
            ThreadServiceImpl.sleepDoInfo = time;
        } else if (syncEnum == SyncEnum.REPORT_INVOICE) {
            ThreadServiceImpl.sleepInvoiceInfo = time;
        } else if (syncEnum == SyncEnum.REPORT_SO) {
            ThreadServiceImpl.sleepSoInfo = time;

        }
        return Message.success(time + "");
    }


    /**
     * @param syncEnum
     * @return
     */
    @ApiOperation("停止线程")
    @GetMapping("/stopThread")
    public Message<String> stopThread(SyncEnum syncEnum) {
        if (syncEnum == SyncEnum.REPORT_SKU) {
            boolean alive = threadserviceImpl.getSyncSkuThread().isAlive();
            if (alive) {
                threadserviceImpl.getSyncSkuThread().interrupt();
            }
        } else if (syncEnum == SyncEnum.REPORT_ADDRESS) {
            boolean alive = threadserviceImpl.getSyncAddressThread().isAlive();
            if (alive) {
                threadserviceImpl.getSyncAddressThread().interrupt();
            }
        } else if (syncEnum == SyncEnum.REPORT_AGENCY) {
            boolean alive = threadserviceImpl.getSyncAddressThread().isAlive();
            if (alive) {
                threadserviceImpl.getSyncAddressThread().interrupt();
            }
        } else if (syncEnum == SyncEnum.REPORT_CS) {
            boolean alive = threadserviceImpl.getSyncCsInfoThread().isAlive();
            if (alive) {
                threadserviceImpl.getSyncCsInfoThread().interrupt();
            }
        } else if (syncEnum == SyncEnum.REPORT_DO) {
            boolean alive = threadserviceImpl.getSycnDoInfoThread().isAlive();
            if (alive) {
                threadserviceImpl.getSycnDoInfoThread().interrupt();
            }
        } else if (syncEnum == SyncEnum.REPORT_INVOICE) {
            boolean alive = threadserviceImpl.getSyncInvoiceInfoThread().isAlive();
            if (alive) {
                threadserviceImpl.getSyncInvoiceInfoThread().interrupt();
            }
        } else if (syncEnum == SyncEnum.REPORT_SO) {
            boolean alive = threadserviceImpl.getSyncSoInfoThread().isAlive();
            if (alive) {
                threadserviceImpl.getSyncSoInfoThread().interrupt();
            }
        }
        return Message.success(null);
    }

    /**
     * @param syncEnum
     * @return
     */
    @ApiOperation("启动线程")
    @GetMapping("/startThread")
    public Message<String> startThread(SyncEnum syncEnum) {
        if (syncEnum == SyncEnum.REPORT_SKU) {
            boolean alive = threadserviceImpl.getSyncSkuThread().isAlive();
            if (!alive) {
                threadserviceImpl.getSyncSkuThread().start();
            }
        } else if (syncEnum == SyncEnum.REPORT_ADDRESS) {
            boolean alive = threadserviceImpl.getSyncAddressThread().isAlive();
            if (!alive) {
                threadserviceImpl.getSyncAddressThread().start();
            }
        } else if (syncEnum == SyncEnum.REPORT_AGENCY) {
            boolean alive = threadserviceImpl.getSyncAddressThread().isAlive();
            if (!alive) {
                threadserviceImpl.getSyncAgencyThread().start();
            }
        } else if (syncEnum == SyncEnum.REPORT_CS) {
            boolean alive = threadserviceImpl.getSyncCsInfoThread().isAlive();
            if (!alive) {
                threadserviceImpl.getSyncCsInfoThread().start();
            }
        } else if (syncEnum == SyncEnum.REPORT_DO) {
            boolean alive = threadserviceImpl.getSycnDoInfoThread().isAlive();
            if (!alive) {
                threadserviceImpl.getSycnDoInfoThread().start();
            }
        } else if (syncEnum == SyncEnum.REPORT_INVOICE) {
            boolean alive = threadserviceImpl.getSyncInvoiceInfoThread().isAlive();
            if (!alive) {
                threadserviceImpl.getSyncInvoiceInfoThread().start();
            }
        } else if (syncEnum == SyncEnum.REPORT_SO) {
            boolean alive = threadserviceImpl.getSyncSoInfoThread().isAlive();
            if (!alive) {
                threadserviceImpl.getSyncSoInfoThread().start();
            }
        }
        return Message.success(null);
    }

    @GetMapping("/startOpenRe")
    @ApiOperation("启动监控")
    public Message<String> startOpenReplicator(Long binlogPosition) {
        openReplicatorRunner.initOpenReplicator();
        if (binlogPosition != null) {
            openReplicatorRunner.getOpenReplicator().setBinlogPosition(binlogPosition);
        }

        openReplicatorRunner.start();
        return Message.success(true + "");
    }

    @GetMapping("/stopOpenReplicator")
    @ApiOperation("停止监控")
    public Message<String> stopOpenReplicator() {
        try {
            openReplicatorRunner.stop();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return Message.success(true + "");
    }


}
